
package require Tcl 8.4     

# Subject: vLib (v0.3) common routines library by v0id

# v0.3
# * putlist_ - bugfix
# * putlist, putlist_, putarray, putarray_ - feature-fix
# + cputmsg -  ,   %...%, %b%, %u%, %r%
# + csubst  -    
# v0.2.4
# + lswap   -     
# * round   -    
# * putmsgx -       ,
#       
# v0.2.3
# + clr0    -   '\003'   
# + reverse -  '\022'   
# + uline   -  '\031'  
# * putmsg  fix -next 
# v0.2.2
# + putctcp -  ctcp-   (   )
# * clr?      "cc,bb", :
#    -  ,
#   bb -   ( 0  15 )
# * timegap     , ..:
#   (1 0  2  = 1  2 )
# * isnum  ''   ( '-'):
#   isnum 123  ;# -> 1
#   isnum -123 ;# -> 0
# * rstrftime      strftime,   
# * putmsgx   3 .  3-   -  
# * duration   rduration   
# v0.2.1
# * parsing routines changed
# + added advanced version of putlist - putlist_
# v0.2.0 (not compatible with 0.1.1)
# + timegap (ex-duration)
# * duration is now fully compatible with egg's one

# Copyright  2005 v0id (refinado el vacho) @ #tcl && #scripting @ WeNet

namespace eval vLib {
  variable const
  array set const {
    c1      "10"
    c2      "04"
    c3      "05"
    c4      "06"
    c5      "07"
  }
  
  # output routines
  namespace export putmsg putmsgx putntc putact putctcp putlist putarray putlist_ putarray_
  # decoring routines
  namespace export csubst cputmsg uline reverse bold clr0 clr1 clr2 clr3 clr4 clr5
  # math routines
  namespace export round 
  # parsing routines
  namespace export isnum isdec ishex isoct isip bsparse isip_
  # time routines
  namespace export timegap duration rctime rstrftime
  # lexical routines
  namespace export numlex
  # variables manipulation routines
  namespace export swap lassign lprepend
}

###############################################################################
# output routines section
###############################################################################
# The routine just provide simple fasion of sending channel/nick messages. It's
# also support '-next' switch for avoiding message queue.
proc ::vLib::putmsg {target text {next ""}} {
  if {$next eq ""} {
    puthelp "PRIVMSG $target :$text"
  } elseif {$next eq "-next"} {
    puthelp "PRIVMSG $target :[lrange $text 0 end-1]" -next
  } else {
    return -code error "wrong args: should be \"putmsg target text ?-next?\""
  }
}
###############################################################################
# cputmsg ?-nocolours -nosubst -nodecor? target text
proc ::vLib::cputmsg {args} {
  if {[llength $args] < 2} { 
    return -code error "wrong # args: should be \"cputmsg ?-nocolours -nodecor? ?-nosubst? target text\""
  }
  set idx 0; set nocolours no; set nosubst no; set nodecor no
  while {[regexp -- {^-(\w+)} [lindex $args $idx] -> option]} {
    incr idx
    switch -exact -- $option {
      nocolours { set nocolours yes }
      nosubst   { set nosubst yes }
      nodecor   { set nodecor yes }
      default   {
        return -code error "wrong args: should be \"cputmsg ?-nocolours -nodecor? ?-nosubst? target text\""
      }
    } ;# switch ...
  } ;# while ...
  if {[llength $args] - $idx != 2} {
    return -code error "wrong # args: should be \"cputmsg ?-nocolours -nodecor? ?-nosubst? target text\""   
  }
  set target [lindex $args $idx]; incr idx
  set text [lindex $args $idx]
  if {$nosubst} {
    puthelp "PRIVMSG $target :$text"
    return -code ok
  }
  if {$nocolours && $nodecor} {
    regsub -all -- {%c(\d{1,2}(,\d{1,2})?)?%} $text {} text
    regsub -all -- {%b%} $text {} text
    regsub -all -- {%u%} $text {} text
    regsub -all -- {%r%} $text {} text
    puthelp "PRIVMSG $target :$text"
    return -code ok
  }
  if {$nocolours} {
    regsub -all -- {%c(\d{1,2}(,\d{1,2})?)?%} $text {} text
    regsub -all -- {%b%} $text "\002" text
    regsub -all -- {%u%} $text "\031" text
    regsub -all -- {%r%} $text "\022" text
    puthelp "PRIVMSG $target :$text"
    return -code ok
  }
  if {$nodecor} {
    regsub -all -- {%c(\d{1,2}(,\d{1,2})?)?%} $text [subst -nobackslashes {[format %c 3]\1}] text
    regsub -all -- {%b%} $text {} text
    regsub -all -- {%u%} $text {} text
    regsub -all -- {%r%} $text {} text
    puthelp "PRIVMSG $target :$text"
    return -code ok
  }
  regsub -all -- {%c(\d{1,2}(,\d{1,2})?)?%} $text [subst -nobackslashes {[format %c 3]\1}] text
  regsub -all -- {%b%} $text "\002" text
  regsub -all -- {%u%} $text "\031" text
  regsub -all -- {%r%} $text "\022" text
  puthelp "PRIVMSG $target :$text"
  return -code ok
}

proc ::vLib::csubst {args} {
  if ![llength $args] { 
    return -code error "wrong # args: should be \"csubst ?-nocolours? ?-nodecor? string\""
  }
  set idx 0; set nocolours no; set nosubst no; set nodecor no
  while {[regexp -- {^-(\w+)} [lindex $args $idx] -> option]} {
    incr idx
    switch -exact -- $option {
      nocolours { set nocolours yes }
      nosubst   { set nosubst yes }
      nodecor   { set nodecor yes }
      default   {
        return -code error "wrong args: should be \"csubst ?-nocolours? ?-nodecor? string\""
      }
    } ;# switch ...
  } ;# while ...
  if {[llength $args] - $idx != 1} {
    return -code error "wrong # args: should be \"csubst ?-nocolours? ?-nodecor? string\""
  }
  set text [lindex $args end]
  if {$nocolours && $nodecor} {
    regsub -all -- {%c(\d{1,2}(,\d{1,2})?)?%} $text {} text
    regsub -all -- {%b%} $text {} text
    regsub -all -- {%u%} $text {} text
    regsub -all -- {%r%} $text {} text
    return $text
  }
  if {$nocolours} {
    regsub -all -- {%c(\d{1,2}(,\d{1,2})?)?%} $text {} text
    regsub -all -- {%b%} $text "\002" text
    regsub -all -- {%u%} $text "\031" text
    regsub -all -- {%r%} $text "\022" text
    return $text
  }
  if {$nodecor} {
    regsub -all -- {%c(\d{1,2}(,\d{1,2})?)?%} $text [subst -nobackslashes {[format %c 3]\1}] text
    regsub -all -- {%b%} $text {} text
    regsub -all -- {%u%} $text {} text
    regsub -all -- {%r%} $text {} text
    return $text
  }
  regsub -all -- {%c(\d{1,2}(,\d{1,2})?)?%} $text [subst -nobackslashes {[format %c 3]\1}] text
  regsub -all -- {%b%} $text "\002" text
  regsub -all -- {%u%} $text "\031" text
  regsub -all -- {%r%} $text "\022" text
  return $text
}

###############################################################################
# This one is for output large chunks of a text. It splits text on a several 
# pieces by specified delimiters, so each piece can be outputed without trancating.
proc ::vLib::putmsgx {target text {lineLen 80} {delims {.,-:?!; }}} {
  if {[string length $text] <= $lineLen} { 
    puthelp "PRIVMSG $target :$text" 
    return
  }
  set _text [split $text $delims]
  set x 0; set i 0
  while {$x < $lineLen} {
    if {$i >= [llength $_text]} { return }
    set wordlen [string length [lindex $_text $i]];
    if {$x + $wordlen > $lineLen} { break }
    incr x $wordlen
    incr x; incr i
  }
  puthelp "PRIVMSG $target :[string range $text 0 [expr $x - 1]]"
  ::vLib::putmsgx $target [string trimleft [string range $text $x end]] $lineLen $delims
}
###############################################################################
# Syntax: 
#   putlist target list ?valPerLine? ?useColours?
# Description:
#   Generic routine for tcl-lists output.
# Parameters:
#   target        - output target. Can be channel or nick;
#   list          - tcl-list for output;
#   ?valPerLine?  - number of list elements in line (default - 0, i.e. all in 1 line);
#   ?useColours?  - use or not colours (default - 0).
# * Probable candidate for removing
###############################################################################
proc ::vLib::putlist {target list {valPerLine 0} {useColours 0}} {
  if {![llength $list]} { return 0 }
  set i 1
  set result {}
  foreach lEntry $list {
    if {$useColours} {
      lappend result "[clr2]$i.[clr1] [join $lEntry]"
    } {
      lappend result "$i. [join $lEntry]"
    }
    incr i
  }
  if {$valPerLine} {
    for {set idx 0} {$idx < [llength $list]} {incr idx $valPerLine} {
      putmsg $target "[join [lrange $result $idx [expr $idx + $valPerLine - 1]]]"
    }
  } {
    putmsg $target "[join $result]"
  }
}
###############################################################################
# Syntax: 
#   putlist_ ?-switches? target list
# Description: 
#   Puts specified list to target. Target may be channel name or nick.
# Switches:
#   -ascii, -dictionary, -integer, -real, -increasing, -decreasing
#     sort switches, see lsort description for information.
#   -perline 'num'
#     specifies number of list-elements per line (default - 0).
#   -colours
#     enables use of colours for output (default - off).
#   -inices
#     enebles use of numeration of list-elements (default - off).
###############################################################################
proc ::vLib::putlist_ {args} {
  if {[llength $args] < 2} {
    return -code error "wrong # args: should be \"putlist_ ?option ...? target list\""
  }
  set list [lindex $args end]; set target [lindex $args end-1]
  set perline 0; set colours 0; set indices 0
  set idx 0; set sortOptions {}
  while {[regexp -- {^-(\w+)$} [lindex $args $idx] -> option]} {
    incr idx
    switch -exact $option {
      ascii -
      dictionary -
      integer -
      real -
      increasing -
      decreasing {
        lappend sortOptions -$option
      }
      perline {
        if [regexp -- {^(\d+)$} [lindex $args $idx] -> value] {
          set perline $value
          incr idx
        } else {
          return -code error "bad value for option 'perline'"
        }
      }
      colours {
        set colours 1
      }
      indices {
        set indices 1
      }
      -- { break }
      default {
        return -code error "bad option (-$option): should be: \"-ascii, -dictionary, -integer, -increasing, -decreasing, \
            -perline, -colours, -indices\""
      }
    } ;# switch...
  } ;# while...
  if {$idx + 2 != [llength $args]} {
    return -code error "wrong # args: should be \"putlist_ ?option ...? target list\""
  }
  if {[llength $sortOptions]} {
    if {[catch {
          set list [eval [subst -novariables {lsort [subst $sortOptions] $list}]]
        }]} {
      return -code error "illegal sort options"
    }
  }
  #putlog "$list"
  set result {}; set i 1
  foreach lEntry $list {
    if {$colours} {
      lappend result [expr {$indices ? "[clr2]$i) [clr1][join $lEntry]" : "[clr1][join $lEntry]"}]
    } {
      lappend result [expr {$indices ? "$i) [join $lEntry]" : "[join $lEntry]"}]
    }
    incr i
  }
  if {$perline} {
    for {set idx 0} {$idx < [llength $list]} {incr idx $perline} {
      if {$idx + $perline - 1 > [llength $list]} {
        putmsg $target "[join [lrange $result $idx end]]"
        break
      }
      putmsg $target "[join [lrange $result $idx [expr $idx + $perline - 1]]]"
    }
  } {
    putmsg $target "[join $result]"
  }
}
###############################################################################
# Mostly generic routine for tcl-arrays output. For params info see putlist
# descrioption above.
# Target array must be a vector (one-dimensional array). This routine sort array
# indices for output using -dictionary option of lsort.
###############################################################################
proc ::vLib::putarray {target arrName {valPerLine 0} {useColours 0}} {
  upvar 1 $arrName arr
  if {![array size arr]} { return 0 }
  set result {}
  foreach idx [lsort -dictionary [array names arr *]] {
    if {$useColours} {
      lappend result "[clr2]$idx[clr1] [join $arr($idx)]"
    } {
      lappend result "$idx [join $arr($idx)]"
    }
  }
  if {$valPerLine} {
    for {set idx 0} {$idx < [array size arr]} {incr idx $valPerLine} {
      putmsg $target "[join [lrange $result $idx [expr $idx + $valPerLine - 1]]]"
    }
  } {
    putmsg $target "[join $result]"
  }
  return 1
}
###############################################################################
# Provides advanced analogue of putarray. Just same fashion as putlist_ except
# using array name instead list as last arg. (See putlist_ description above)
###############################################################################
proc ::vLib::putarray_ {args} {
  if {[llength $args] < 2} {
    return -code error "wrong # args: should be \"?option ...? target arrayName\""
  }
  upvar 1 [lindex $args end] vector
  set target [lindex $args end-1]
  set perline 0; set colours 0; set indices 0
  set idx 0; set sortOptions {}
  while {[regexp -- {^-(\w+)$} [lindex $args $idx] -> option]} {
    incr idx
    switch -exact $option {
      ascii -
      dictionary -
      integer -
      real -
      increasing -
      decreasing {
        lappend sortOptions -$option
      }
      perline {
        if [regexp -- {^(\d+)$} [lindex $args $idx] -> value] {
          set perline $value
          incr idx
        } else {
          return -code error "bad value for option 'perline'"
        }
      }
      colours {
        set colours 1
      }
      indices {
        set indices 1
      }
      -- { break }
      default {
        return -code error "bad option (-$option): should be \"-ascii, -dictionary, -integer, -increasing, -decreasing, \
            -perline, -colours, -indices\""
      }
    } ;# switch...
  } ;# while...
  if {$idx + 2 != [llength $args]} {
    return -code error "wrong # args: should be \"?option ...? target arrayName\""
  }
  set list [array names vector *]
  if {[llength $sortOptions]} {
    if {[catch {
          set list [eval [subst -novariables {lsort [subst $sortOptions] $list}]]
        }]} {
      return -code error "illegal sort options"
    }
  }
  #putlog "$list"
  set result {}; set i 1
  foreach index $list {
    if {$colours} {
      lappend result [expr {$indices ? "[clr2]$index) [clr1][join $vector($index)]" : "[clr1][join $vector($index)]"}]
    } {
      lappend result [expr {$indices ? "$index) [join $vector($index)]" : "[join $vector($index)]"}]
    }
    incr i
  }
  if {$perline} {
    for {set idx 0} {$idx < [array size vector]} {incr idx $perline} {
      if {($idx + $perline - 1) > [llength $list]} {
        putmsg $target "[join [lrange $result $idx end]]"
        break
      }
      putmsg $target "[join [lrange $result $idx [expr $idx + $perline - 1]]]"
    }
  } {
    putmsg $target "[join $result]"
  }
}
###############################################################################
proc ::vLib::putntc {target text} { puthelp "NOTICE $target :$text" }
###############################################################################
proc ::vLib::putact {target text} { puthelp "PRIVMSG $target :\001ACTION $text" }
###############################################################################
proc ::vLib::putctcp {target text} {  puthelp "PRIVMSG $target :\001$text" }
###############################################################################
proc ::vLib::bold {}    { return [format "%c" 2] }
###############################################################################
proc ::vLib::reverse {} { return [format "%c" 22] }
###############################################################################
proc ::vLib::uline {}   { return [format "%c" 31] }
###############################################################################
proc ::vLib::clr0 {}    { return [format "%c" 3] }
###############################################################################
proc ::vLib::clr1 {{clr ""}} {
  variable const
  if {$clr ne ""} {
    if {![regexp -- {^(\d{1,2})(?:,(\d{1,2}))?$} $clr -> fc bc]} {
      return -code error "wrong colour value"
    }
    if {$fc > 15 || ($bc ne "" && $bc > 15)} {
      return -code error "wrong colour value"
    }
    set const(c1) $clr
  }
  return "\003$const(c1)"
}
###############################################################################
proc ::vLib::clr2 {{clr ""}} {
  variable const
  if {$clr ne ""} {
    if {![regexp -- {^(\d{1,2})(?:,(\d{1,2}))?$} $clr -> fc bc]} {
      return -code error "wrong colour value"
    }
    if {$fc > 15 || ($bc ne "" && $bc > 15)} {
      return -code error "wrong colour value"
    }
    set const(c2) $clr
  }
  return "\003$const(c2)"
}
###############################################################################
proc ::vLib::clr3 {{clr ""}} {
  variable const
  if {$clr ne ""} {
    if {![regexp -- {^(\d{1,2})(?:,(\d{1,2}))?$} $clr -> fc bc]} {
      return -code error "wrong colour value"
    }
    if {$fc > 15 || ($bc ne "" && $bc > 15)} {
      return -code error "wrong colour value"
    }
    set const(c3) $clr
  }
  return "\003$const(c3)"
}
###############################################################################
proc ::vLib::clr4 {{clr ""}} {
  variable const
  if {$clr ne ""} {
    if {![regexp -- {^(\d{1,2})(?:,(\d{1,2}))?$} $clr -> fc bc]} {
      return -code error "wrong colour value"
    }
    if {$fc > 15 || ($bc ne "" && $bc > 15)} {
      return -code error "wrong colour value"
    }
    set const(c4) $clr
  }
  return "\003$const(c4)"
}
###############################################################################
proc ::vLib::clr5 {{clr ""}} {
  variable const
  if {$clr ne ""} {
    if {![regexp -- {^(\d{1,2})(?:,(\d{1,2}))?$} $clr -> fc bc]} {
      return -code error "wrong colour value"
    }
    if {$fc > 15 || ($bc ne "" && $bc > 15)} {
      return -code error "wrong colour value"
    }
    set const(c5) $clr
  }
  return "\003$const(c5)"
}
###############################################################################
# Syntax:
#   round num ?prec?
# Description
#   Round off a number to a given precision and return the result
# * Grabbed from math::fuzzy::troundn @ tcllib1.7
###############################################################################
proc ::vLib::round {num {prec 0}} {
  set scale [expr {pow(10.0, $prec)}]
  set rounded [expr {floor($num * $scale + 0.5)}]
  return [expr {$rounded / $scale}]
}
###############################################################################
proc ::vLib::isdec {text} {
  return [regexp {(?i)^-?\d*(\.\d+)?(\Ye+[+-]?\d+)?$} $text]
}
###############################################################################
proc ::vLib::ishex {text} {
  return [regexp {(?i)^-?0x[0-9A-F]+$} $text]
}
###############################################################################
proc ::vLib::isoct {text} {
  return [regexp {^-?0[0-7]+$} $text]
}
###############################################################################
proc ::vLib::isnum {text} {
  return [regexp {^\d+$} $text]
}
###############################################################################
proc ::vLib::isip_ {text} {
  if {[regexp -- {^(\d{1,3}\.){3}\d{1,3}$} $text]} {
    set i 0
    foreach n [split $text "."] {
      if {($i <= 2 && $n > 255) ||
          ($i == 3 && ($n > 254 || $n < 1))} { return 0 }
      incr i
    }
    return 1
  } 
  if {[regexp -nocase -- {^([0-9A-F]{1,2}\.){3}[0-9A-F]{1,2}$} $text]} {
    set i 0   
    foreach n [split $text "."] {
      set n "0x$n"
      if {($i <= 2 && $n > 255) || 
          ($i == 3 && ($n > 254 || $n < 1))} { return 0 }
      incr i
    }
    return 1
  }
  return 0
}
###############################################################################
proc ::vLib::isip {text} {
  if {[regexp -- {^(\d+)\.(\d+)\.(\d+)\.(\d+)$} $text -> a b c d]} {
    if {$a >= 0 && $a < 256 && 
        $b >= 0 && $a < 256 && 
        $c >= 0 && $a < 256 && 
        $d > 0 && $d < 255} { return 1 }
  }
  return 0
}
###############################################################################
# Doubles backslashes ('\') for correct interpretation.
# Must be slow enough because of exhaustive search. It's not a good idea to use
# it for parsing large chunks of text.
proc ::vLib::bsparse {text} {
  set result ""
  foreach char [split $text {}] {
    if {[scan $char %c] == 92} {
      append result [format "%c%c" 92 92]
      continue
    }
    append result $char
  }
  return $result
}
###############################################################################
# Syntax:
#   lassign list varName ?varName2 varName3...?
# Description:
#   assigns list values to specified variables respectively, so first value of
#   list will assigned to the first variable, second - to the second and so on.
#   The number of variables must be less then or equal to length of given list.
###############################################################################
proc ::vLib::lassign {args} {
  if {[llength $args] < 2} {
    return -code error "wrong # args: should be \"lassign list varName ?varName varName ...?\""
  }
  set list [lindex $args 0]
  set vars [lreplace $args 0 0]
  set idx 0
  foreach var $vars {
    upvar 1 $var var_
    if {$idx < [llength $list]} {
      set var_ [lindex $list $idx]
      incr idx
    } {
      return -code error "Have not enough list elements"
    }
  }
}
###############################################################################
proc ::vLib::timegap {start {end ""}} {
  set end [expr {($end eq "") ? [unixtime] : $end}]
  set gap [expr {($start > $end) ? [expr $start - $end] : [expr $end - $start]}]  
  set result {}
  # less than 1 minute?
  if {$gap < 60} {
    return "$gap [numlex $gap {  }]"
  }
  # more than 59 minutes?
  if {[expr $gap / 60] >= 60} {
    # more than 23 hours?
    if {[expr $gap / 3600] >= 24} {
      # more than 6 days?
      if {[expr $gap / 86400] >= 7} {
        set tmp [expr int($gap / 606800)]
        set gap [expr $gap % 606800]
        lappend result "$tmp [numlex $tmp {  }]"
      }
      if {[set tmp [expr int($gap / 86400)]]} {
        set gap [expr $gap % 86400]
        lappend result "$tmp [numlex $tmp {  }]"
      }
    }
    if {[set tmp [expr int($gap / 3600)]]} {
      set gap [expr $gap % 3600]
      lappend result "$tmp [numlex $tmp {  }]"
    }
  }
  if {[set tmp [expr int($gap / 60)]]} {
    set gap [expr $gap % 60]
    lappend result "$tmp [numlex $tmp {  }]"
  }
  if {$gap} {
    lappend result "$gap [numlex $gap {  }]"
  }
  return [join $result]
}
###############################################################################
proc ::vLib::rduration {seconds} {
  if {![isdec $seconds]} {
    return "0 "
  }
  return [timegap [unixtime] [expr {[unixtime] + $seconds}]]
}
###############################################################################
proc ::vLib::rctime {unixTime} {
  if {[catch {
    set result [ctime $unixTime]
  }]} {
    return ""
  }
  set result [string map {
    Mon  Tue  Wed  Thu  Fri  Sat  Sun 
  } $result]
  set result [string map {
    Jan  Feb  Mar  Apr  May  Jun  Jul  Aug  Sep  Oct  Nov  Dec 
  } $result]
  return $result
}
############################################################################### 
proc ::vLib::rstrftime {formatStr {time ""}} {
  if {$time eq ""} { set time [unixtime] }
  if {[catch { 
    set result [strftime $formatStr $time]
  } err]} {
    return $err
  }
  # Abbreviated weekday names
  if {([string first "%a" $formatStr] != -1) || ([string first "%x" $formatStr] != -1)} {
    set result [string map {
      Mon  Tue  Wed  Thu  Fri  Sat  Sun 
    } $result]
  }
  # Abbreviated month names
  if {([string first "%b" $formatStr] != -1) || ([string first "%x" $formatStr] != -1)} {
    set result [string map {
      Jan  Feb  Mar  Apr  May  Jun  Jul  Aug  Sep  Oct  Nov  Dec 
    } $result]
  }
  # Full weekday names
  if {[string first "%A" $formatStr] != -1} {
    set result [string map {
      Monday  Tuesday  Wednesday  \
      Thursday  Friday  Saturday  Sunday 
    } $result]
  }
  # Full month names
  if {[string first "%B" $formatStr] != -1} {
    set result [string map {
      January  February  March  April  \
      May  June  July  August  September  \
      October  November  December 
    } $result]
  }
  return $result
}
###############################################################################
# * This one is useless for not russian-localized scripts
proc ::vLib::numlex {num lex} {
  if {[llength $lex] != 3} { return "" }
  set num [expr int($num)]
  if {[string length $num] > 2} { set num [string range $num end-1 end] }
  if {($num > 10) && ($num < 21)} { return [lindex $lex 2] }
  set num [string index $num end]
  switch -exact -- $num {
    1 { return [lindex $lex 0] }
    2 { return [lindex $lex 1] }
    3 { return [lindex $lex 1] }
    4 { return [lindex $lex 1] }
    default { return [lindex $lex 2] }
  }
}
###############################################################################
# Syntax:
#   swap varName1 varName2
# Description:
#   Swaps values of specified variables. Take note of the syntax, it's the same
#   as in 'set' or 'incr' commands. E.g. you must use 'swap a b', not 'swap $a $b'
#   for swapping values of those two variables.
###############################################################################
proc ::vLib::swap {a b} {
  upvar 1 $a a_
  upvar 1 $b b_
  set c   $a_
  set a_  $b_
  set b_  $c
}
###############################################################################
# Syntax:
#   lprepend varName ?value value value...?
# Description:
#   It's lappend analogue, except the position for insertion. In this one
#   specified values would be inserted to the begin of list.
# * Was grabbed from Tcl8.4/Tk8.4 Manual
###############################################################################
proc ::vLib::lprepend {varName args} {
   upvar 1 $varName var
   # Ensure that the variable exists and contains a list
   lappend var
   # Now we insert all the arguments in one go
   set var [eval [list linsert $var 0] $args]
}
###############################################################################
# Syntax:
#   lswap listName i j
# Description:
#   Exchange two elements of a list
# * Was grabbed from struct::list::Lswap @ tcllib 1.7
###############################################################################
proc ::vLib::lswap {listName i j} {
  upvar $listName list
  set len [llength $list]
  if {($i < 0) || ($j < 0) ||
      ($i >= $len) || ($j >= $len)} {
    return -code error "list index is out of range"
  }
  if {$i != $j} {
    set tmp [lindex $list $i]
    lset list $i [lindex $list $j]
    lset list $j $tmp
  }
  return $list
}
###############################################################################
package provide vLib 0.3
